package com.refinitiv.dss.api.example;
/*
 * ====================================================================
 * This Java example demonstrate how to make an HTTPS request to DSS2
 * to make a historical search.
 * 
 * To run this example, you must provide a DSS2 username and password.
 * ====================================================================  
 */

import java.io.IOException;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.util.Scanner;

import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.client.BasicResponseHandler;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.ResponseHandler;
import org.json.JSONObject;

public class DSSHistoricalSearchRTH {
    private String urlHost = "https://hosted.datascopeapi.reuters.com/RestApi/v1";
    private String sessionToken = "";
    private CloseableHttpClient httpclient = HttpClientBuilder.create().disableContentCompression().build();

    public static void main(String[] args) throws Exception {

        if(args.length < 2) {
            System.out.println("Please enter DSS2 Username as 1st command line parameter, Password as 2nd");
            System.exit(-1);
        }

        DSSHistoricalSearchRTH dss2 = new DSSHistoricalSearchRTH();
        dss2.getSessionToken(args[0], args[1]);
        System.out.println("");

        // Search by ISIN Request 
        dss2.historicalSearchRequest();

        dss2.httpclient.close();
    }


    /**
     * Request DSS2 for a Session Token (24 hour life)
     * 
     * @param username
     * @param password
     */
    public void getSessionToken(String username, String password) {

        try {
            HttpPost httppost = new HttpPost(urlHost + "/Authentication/RequestToken");

            httppost.addHeader("content-type", "application/json; charset=UTF-8");

            JSONObject TokenRequest = new JSONObject()
                .put("Credentials", new JSONObject()
                    .put("Username",  username)
                    .put("Password",  password));

            StringEntity requestBody = new StringEntity(TokenRequest.toString());

            httppost.setEntity(requestBody);

            ResponseHandler<String> responseHandler = new BasicResponseHandler();

            String response = httpclient.execute(httppost, responseHandler);
            JSONObject jsonResponse = new JSONObject(response);

            sessionToken = jsonResponse.get("value").toString();
            System.out.println("Session Token (expires in 24 hours):\n" + sessionToken);

        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    public void historicalSearchRequest() {

        try {
            HttpPost httppost = new HttpPost(urlHost + "/Search/HistoricalSearch"); 

            httppost.addHeader("content-type", "application/json;odata.metadata=minimal");
            httppost.addHeader("Authorization", "Token " + sessionToken);

            JSONObject searchJSONObject = new JSONObject()
                .put("Request", new JSONObject()
                    .put("IdentifierType", "Isin")
                    .put("Identifier", "US4592001014")
                    .put("Range", new JSONObject()
                        .put("Start", "2008-01-01T00:00:00.000Z")
                        .put("End", "2008-01-01T00:00:00.000Z")));

            System.out.println("Historical search JSON request content:\n"+ searchJSONObject.toString());
            StringEntity requestBody = new StringEntity(searchJSONObject.toString());

            httppost.setEntity(requestBody);

            ResponseHandler<String> responseHandler = new BasicResponseHandler();

            String response = httpclient.execute(httppost, responseHandler);
            JSONObject jsonResponse = new JSONObject(response);

            StringWriter out = new StringWriter();
            jsonResponse.write(out);
            System.out.println("\nJSON response contents:\n"+ out);

        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    public static void promptEnterKey() {
        System.out.println("Press \"ENTER\" to continue...");
        Scanner scanner = new Scanner(System.in);
        scanner.nextLine();
    }
} //EOF
