package com.refinitiv.dss.api.example;
/*
 * ====================================================================
 * This Java example demonstrate how to make an HTTPS request to DSS2
 * to retrieve a Session Token.
 * 
 * To run this example, you must provide a DSS2 username and password.
 * ====================================================================
 */

import java.io.IOException;
import java.io.UnsupportedEncodingException;

import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.client.BasicResponseHandler;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.ResponseHandler;
import org.json.JSONObject;

public class DSSToken {
    private String urlHost = "https://hosted.datascopeapi.reuters.com/RestApi/v1";
    private String sessionToken = "";
    private CloseableHttpClient httpclient = HttpClientBuilder.create().disableContentCompression().build();

    public static void main(String[] args) throws Exception {

        if(args.length < 2) {
            System.out.println("Please enter DSS2 Username as 1st command line parameter, Password as 2nd");
            System.exit(-1);
        }

        DSSToken dss2 = new DSSToken();
        dss2.getSessionToken(args[0], args[1]);

        dss2.httpclient.close();
    }


    /**
     * Request DSS2 for a Session Token (24 hour life)
     * 
     * @param username
     * @param password
     */
    public void getSessionToken(String username, String password) {

        try {
            HttpPost httppost = new HttpPost(urlHost + "/Authentication/RequestToken");

            httppost.addHeader("content-type", "application/json; charset=UTF-8");

            JSONObject TokenRequest = new JSONObject()
                .put("Credentials", new JSONObject()
                    .put("Username",  username)
                    .put("Password",  password));

            StringEntity requestBody = new StringEntity(TokenRequest.toString());

            httppost.setEntity(requestBody);

            ResponseHandler<String> responseHandler = new BasicResponseHandler();

            String response = httpclient.execute(httppost, responseHandler);
            JSONObject jsonResponse = new JSONObject(response);

            sessionToken = jsonResponse.get("value").toString();
            System.out.println("Session Token (expires in 24 hours):\n" + sessionToken);

        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
       
} //EOF
