package com.refinitiv.dss.api.example;

/*
 * ====================================================================
 * This Java example demonstrate how to make an HTTPS request to DSS2
 * to retrieve a Session Token. This sample is written for use with a
 * proxy server.
 * 
 * To run this example, you must provide a DSS2 username, password and
 * the proxy server hostname and proxy port.
 * ====================================================================
 */

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;

import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.client.BasicResponseHandler;
import org.apache.http.client.*;
import org.apache.http.*;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.conn.DefaultProxyRoutePlanner;
import org.json.JSONException;
import org.json.JSONObject;

public class DSSTokenProxy {
    private static final String urlHost = "https://hosted.datascopeapi.reuters.com/RestApi/v1";
    private String sessionToken = "";
    private CloseableHttpClient httpclient = HttpClientBuilder.create().disableContentCompression().build();

    public static void main(String[] args) throws Exception {

        if(args.length < 4) {
            System.out.println("Please enter DSS2 Username, DSS2 Password, Proxy Hostname, Proxy Port");
            System.exit(-1);
        }

		int pPort = Integer.parseInt(args[3]);

        DSSTokenProxy dss2 = new DSSTokenProxy();
        dss2.getSessionToken(args[2], pPort, args[0], args[1]);

        dss2.httpclient.close();
    }


    /**
     * Request DSS2 for a Session Token (24 hour life)
     * 
     * @param proxyHostName
     * @param proxyPortNumber
     * @param username
     * @param password
     */
    public void getSessionToken(String proxyHostName, int proxyPortNumber, String username, String password) {

        try {
			// Set proxy information for HTTP Components
			HttpHost proxy = new HttpHost(proxyHostName, proxyPortNumber);
			DefaultProxyRoutePlanner routePlanner = new DefaultProxyRoutePlanner(proxy);
			CloseableHttpClient httpclient = HttpClients.custom()
				.setRoutePlanner(routePlanner)
				.build();

			// Issue a POST request
			HttpPost httppost = new HttpPost(urlHost + "/Authentication/RequestToken");
            httppost.addHeader("content-type", "application/json; charset=UTF-8");

            JSONObject TokenRequest = new JSONObject()
                .put("Credentials", new JSONObject()
                    .put("Username",  username)
                    .put("Password",  password));

            StringEntity requestBody = new StringEntity(TokenRequest.toString());

            httppost.setEntity(requestBody);

            ResponseHandler<String> responseHandler = new BasicResponseHandler();

            String response = httpclient.execute(httppost, responseHandler);
            JSONObject jsonResponse = new JSONObject(response);

            sessionToken = jsonResponse.get("value").toString();
            System.out.println("Session Token (expires in 24 hours):\n" + sessionToken);

        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
       
} //EOF
