package com.refinitiv.dss.api.example;

/*
 * ====================================================================
 * This Java example demonstrate how to make an HTTPS request to DSS2
 * to retrieve a Session Token. This sample is written for use with a
 * proxy server that requires authentication.
 * 
 * To run this example, you must provide a DSS2 username and password and
 * the proxy server hostname, port, username and password.
 * ====================================================================
 */

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;

import org.apache.http.client.methods.HttpPost;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.conn.ssl.TrustSelfSignedStrategy;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.apache.http.impl.client.BasicResponseHandler;
import org.apache.http.client.*;
import org.apache.http.*;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.conn.DefaultProxyRoutePlanner;
import org.apache.http.ssl.SSLContextBuilder;
import org.json.JSONException;
import org.json.JSONObject;

public class DSSTokenProxyAuth {
    private static final String urlHost = "https://hosted.datascopeapi.reuters.com/RestApi/v1";
    private String sessionToken = "";
    private CloseableHttpClient httpclient = HttpClientBuilder.create().disableContentCompression().build();

    public static void main(String[] args) throws Exception {

        if(args.length < 6) {
            System.out.println("Please enter DSS2 Username, DSS2 Password, Proxy Hostname, Proxy Port, Proxy Username, Proxy Password");
            System.exit(-1);
        }

		int pPort = Integer.parseInt(args[3]);

        DSSTokenProxyAuth dss2 = new DSSTokenProxyAuth();
        dss2.getSessionTokenProxyAuth(args[2], pPort, args[4], args[5], args[0], args[1]);

        dss2.httpclient.close();
    }


    /**
     * Request DSS2 for a Session Token (24 hour life), via proxy with authentication
     * 
     * @param proxyHostName
     * @param proxyPortNumber
     * @param proxyUserName
     * @param proxyPassword
     * @param username
     * @param password
     */
    public void getSessionTokenProxyAuth(String proxyHostName, int proxyPortNumber, String proxyUserName, String proxyPassword,
    		String username, String password) {

        try {
    	    // Set proxy information for HTTP Components
			HttpHost proxy = new HttpHost(proxyHostName, proxyPortNumber);
			DefaultProxyRoutePlanner routePlanner = new DefaultProxyRoutePlanner(proxy);
			CredentialsProvider credsProvider = new BasicCredentialsProvider();
			credsProvider.setCredentials(new AuthScope(proxyHostName, proxyPortNumber, AuthScope.ANY_REALM),
			    new UsernamePasswordCredentials(proxyUserName, proxyPassword));
			SSLContextBuilder sslContextBuilder = new SSLContextBuilder();
			sslContextBuilder.loadTrustMaterial(null, new TrustSelfSignedStrategy());
			SSLConnectionSocketFactory sslsf = new SSLConnectionSocketFactory(sslContextBuilder.build());
			httpclient = HttpClients.custom()
			    .setSSLSocketFactory(sslsf)
			    .setRoutePlanner(routePlanner)
			    .setDefaultCredentialsProvider(credsProvider)
			    .build();

			// Issue a POST request
			HttpPost httppost = new HttpPost(urlHost + "/Authentication/RequestToken");
            httppost.addHeader("content-type", "application/json; charset=UTF-8");

            JSONObject TokenRequest = new JSONObject()
                .put("Credentials", new JSONObject()
                    .put("Username",  username)
                    .put("Password",  password));

            StringEntity requestBody = new StringEntity(TokenRequest.toString());

            httppost.setEntity(requestBody);

            ResponseHandler<String> responseHandler = new BasicResponseHandler();

            String response = httpclient.execute(httppost, responseHandler);
            JSONObject jsonResponse = new JSONObject(response);

            sessionToken = jsonResponse.get("value").toString();
            System.out.println("Session Token (expires in 24 hours):\n" + sessionToken);

        } catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		} catch (KeyStoreException e) {
			e.printStackTrace();
		} catch (KeyManagementException e) {
			e.printStackTrace();
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
		}
    }
       
} //EOF
